!----------------------------------------------------------------------------------------------------------------------------------!
SUBROUTINE cbem_lap_assem
!----------------------------------------------------------------------------------------------------------------------------------!

!===purpose===! 

!**********************************************************************************************************************************!
!********************************  COLLOCATION BEM for 3D LAPLACE EQUATION: SINGULAR FORMULATION  *********************************!
!***************************************  3-NODED TRIANGULAR ELEMENT: ANALYTIC INTEGRATION  ***************************************!
!***********************************************  PIECEWISE CONSTANT APPROXIMATION  ***********************************************!
!********************************************  Assemble h and g influence matrices  ***********************************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata
!*** Version:   V.1.3
!*** Released:  Jun 14, 2012

 
USE KINDS
USE SRF_GEOM, ONLY: nbelt ! nbelt == Total number of boundary elements
                          ! For Piecewise constant approximation, nbelt is also equal to the number of collocation points on the surface
USE DISCR_BIE,ONLY: h,g   ! h == Coefficient matrix acting on the vector of boundary potentials <==> potential influence matrix
                          ! g == Coefficient matrix acting on the vector of boundary fluxes <==> flux influence matrix


IMPLICIT NONE

 integer(kind=k9) :: k,i,alloc_st



    allocate(h(nbelt,nbelt), g(nbelt,nbelt),STAT=alloc_st)
    if ( alloc_st == 0 ) then ! If h,g are successfully allocated
          loop0a: do k=1,nbelt
	     loop0b: do i=1, nbelt
                h(i,k) = 0.0_dp; g(i,k) = 0.0_dp
             enddo loop0b
          enddo loop0a

!*** Assemble h and g influence matrices
          loop1a: do k=1,nbelt ! Loop over all boundary elements
	     call cbem_lap_ex_int_ud(k) ! For every triangle k, compute and add the influence contribution to all collocation points in h and g
          enddo loop1a
    else
          write(*,*)'cbem_lap_assem: Allocate arrays "h,g" failed with Status=',alloc_st
	  stop
    endif

!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE cbem_lap_assem
!----------------------------------------------------------------------------------------------------------------------------------!
