!----------------------------------------------------------------------------------------------------------------------------------!
SUBROUTINE cbem_lap_srf_eval(InfParams,InfData,tm0)
!----------------------------------------------------------------------------------------------------------------------------------!

!===purpose===! 

!**********************************************************************************************************************************!
!********************************  COLLOCATION BEM for 3D LAPLACE EQUATION: SINGULAR FORMULATION  *********************************!
!***************************************  3-NODED TRIANGULAR ELEMENT: ANALYTIC INTEGRATION  ***************************************!
!***********************************************  PIECEWISE CONSTANT APPROXIMATION  ***********************************************!
!***************************************  Solve the Singular BIE for the Laplace equation  ****************************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata
!*** Version:   V.1.3
!*** Released:  Jun 14, 2012

 
USE KINDS


IMPLICIT NONE

 real(kind=dp), intent(in) :: tm0 ! Initial time

 character(len=72) :: InfParams,InfData
 real(kind=dp) :: tm1 ! time variable



    write(*,100)' >>>  COLLOCATION BEM for 3D LAPLACE EQUATION: SINGULAR FORMULATION ==> CBEM_LAP <== Version 1.3  <<<'

    write(*,100)' >>>  Data Input & Initializations  <<<'

     call cbem_lap_input(InfParams,InfData) ! Read input data and parameters

     call cbem_lap_init ! Initialize arrays and parameters

    call cpu_time(tm1) ! Current time
    write(*,*)'Time == ',tm1-tm0,'s'

    write(*,100)' >>>  Assembly of the Discretized BIE  <<<'   

     call cbem_lap_assem ! Assemble h and g influence matrices

    call cpu_time(tm1) ! Current time
    write(*,*)'Time == ',tm1-tm0,'s'

    write(*,100)' >>>  Left-hand side & Right-hand side of the Discretized BIE  <<<'

     call cbem_lap_lrhs ! Form the left- and right-hand sides of the discretized BIE

    call cpu_time(tm1) ! Current time
    write(*,*)'Time == ',tm1-tm0,'s'
  
    write(*,100)' >>>  Solution of the Discretized BIE  <<<'          

     call cbem_lap_solv ! Solve the Discretized BIE


  100 format (1x, a)

!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE cbem_lap_srf_eval
!----------------------------------------------------------------------------------------------------------------------------------!
