
!**********************************************************************************************************************************!
!***************************************************  Useful routines for CBEM  ***************************************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata
!*** Version:   V.1.3
!*** Released:  Jun 14, 2012


!----------------------------------------------------------------------------------------------------------------------------------! 
FUNCTION gammai(p1,p2,qet,cr1,cr2,zn,zd)
!----------------------------------------------------------------------------------------------------------------------------------!	

!===purpose===! 

!**********************************************************************************************************************************!
!***************************************************  Determine gamma_i  **********************************************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata, Oak Ridge National Laboratory
!*** Version:   V.1.2
!*** Released:  Apr 14, 2011


USE KINDS
USE PARAMS, ONLY: pix2 ! pix2 = 2*pi, pi = 3.1415926535897932


IMPLICIT NONE

 real   (kind=dp), intent(in) :: p1,p2,qet,cr1,cr2,zn,zd
 real   (kind=dp)             :: gammai



	     if ( (p1<0.0_dp) .and. (p2>0.0_dp) ) then ! 1-----x----------2 <==> if Projection of collocation point falls in side k

		         if ( qet<0.0_dp ) then

	                      if ( (cr1>0.0_dp) .and. (cr2<0.0_dp) .and. (zn>0.0_dp) .and. (zd<0.0_dp) ) then
		                gammai = datan2(zn,zd) - pix2
		              elseif ( (cr1<0.0_dp) .and. (cr2>0.0_dp) .and. (zn>0.0_dp) .and. (zd<0.0_dp) ) then
		                gammai = datan2(zn,zd) - pix2
		              elseif ( (cr1<0.0_dp) .and. (cr2<0.0_dp) .and. (zn>0.0_dp) ) then
		                gammai = datan2(zn,zd) - pix2
		              else
		                gammai = datan2(zn,zd)
		              endif
		      
		         elseif ( qet>0.0_dp ) then

	                      if ( (cr1>0.0_dp) .and. (cr2<0.0_dp) .and. (zn<0.0_dp) .and. (zd<0.0_dp) ) then
		                 gammai = datan2(zn,zd) + pix2
		              elseif ( (cr1<0.0_dp) .and. (cr2>0.0_dp) .and. (zn<0.0_dp) .and. (zd<0.0_dp) ) then
		                 gammai = datan2(zn,zd) + pix2
		              elseif ( (cr1<0.0_dp) .and. (cr2<0.0_dp) .and. (zn<0.0_dp) ) then
		                 gammai = datan2(zn,zd) + pix2
		              else
		                gammai = datan2(zn,zd)
		              endif
		      
		         else
		            gammai = 0.0_dp    ! gammai = datan2(zn,zd) = 0.0
		         endif  
		
	     
	     else !  x 1----------------2 or 1----------------2 x <==> if Projection of collocation point does not fall in side k
	       gammai = datan2(zn,zd) 
	     endif

!----------------------------------------------------------------------------------------------------------------------------------! 
END FUNCTION gammai
!----------------------------------------------------------------------------------------------------------------------------------!	


!----------------------------------------------------------------------------------------------------------------------------------! 
FUNCTION pt_tria_tst(w,u,v,theta,theta_id)
!----------------------------------------------------------------------------------------------------------------------------------!	

!===purpose===! 

!**********************************************************************************************************************************!
!*************************************************  Point in Triangle Test  *******************************************************!
!***********************  Given a point with barycentric coordinates (u,v) on a plane containing a triangle  **********************!
!************  Determine whether the point (u,v) is outside, or inside, or on an edge, or at a vertex of the triangle  ************!
!**************************  Set the flag theta_id characterizing the position of the query point (u,v)  **************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata, Oak Ridge National Laboratory
!*** Version:   V.1.2
!*** Released:  Apr 14, 2011


!*** Triangle with vertices {y1,y2,y3}

!           y3
!	    *
!	   / \
!	  /   \
!	 /     \
!       /       \
!      /	 \
!     /		  \
!    *-------------*
!  y1               y2

! Oriented edge L1 = [y1,y2] <== Edge1
! Oriented edge L2 = [y2,y3] <== Edge2
! Oriented edge L3 = [y3,y1] <== Edge3


USE KINDS
USE PARAMS, ONLY: pi,pix2 ! pi = 3.1415926535897932, pix2 = 2*pi


IMPLICIT NONE

 integer(kind=k9), intent(out) :: theta_id
 real   (kind=dp), intent(in ) :: w,u,v
 real   (kind=dp), intent(in ) :: theta(3)
 real   (kind=dp)              :: pt_tria_tst
!*** Note: (u,v) == Barycentric coordinates of the query point on the plane containing the triangle
!*** Note: w = 1-(u+v)



     theta_id = -1

     if ( u<0.0_dp .or. v<0.0_dp .or. w<0.0_dp ) then
	 theta_id = 7; pt_tria_tst = 0.0_dp ! Query point is Outside the triangle
     elseif ( u==0.0_dp ) then
           if ( v==0.0_dp ) then
              theta_id = 4; pt_tria_tst = theta(1) ! Query point is at Vertex1
	   elseif ( v==1.0_dp ) then
              theta_id = 6; pt_tria_tst = theta(3) ! Query point is at Vertex3
	   elseif ( v<0.0_dp .or. v>1.0_dp ) then
	      theta_id = 7; pt_tria_tst = 0.0_dp ! Query point is Outside the triangle
	   else ! v>0.0_dp & v<1.0_dp
	      theta_id = 3; pt_tria_tst = pi ! Query point is on Edge3 of the triangle
	   endif
     elseif ( v==0.0_dp ) then
	   if ( u==1.0_dp ) then
              theta_id = 5; pt_tria_tst = theta(2) ! Query point is at Vertex2
	   elseif ( u<0.0_dp .or. u>1.0_dp ) then
	      theta_id = 7; pt_tria_tst = 0.0_dp ! Query point is Outside the triangle
	   else ! u>0.0_dp & u<1.0_dp
	      theta_id = 1; pt_tria_tst = pi ! Query point is on Edge1 of the triangle
	   endif
     elseif ( w==0.0_dp ) then
	   if ( u<0.0_dp .or. v<0.0_dp ) then
	      theta_id = 7; pt_tria_tst = 0.0_dp ! Query point is Outside the triangle
	   else ! u>0.0_dp & v>0.0_dp
	      theta_id = 2; pt_tria_tst = pi ! Query point is on Edge2 of the triangle
	   endif   
     else ! u>0.0_dp & v>0.0_dp & w>0.0_dp
	theta_id = 0; pt_tria_tst = pix2 ! Query point is Inside the triangle
     endif


!----------------------------------------------------------------------------------------------------------------------------------! 
END FUNCTION pt_tria_tst
!----------------------------------------------------------------------------------------------------------------------------------!	


!----------------------------------------------------------------------------------------------------------------------------------!
FUNCTION asym_chi(p1,p2,d)
!----------------------------------------------------------------------------------------------------------------------------------!

!===purpose===! 

!**********************************************************************************************************************************!
!************************************  return ASYMTOTIC VALUE of chi when q^2 + et^2 = d << 1  ************************************!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata, Oak Ridge National Laboratory
!*** Version:   V.1.2
!*** Released:  Apr 14, 2011

 
USE KINDS


IMPLICIT NONE

 real(kind=dp), intent(in) :: p1,p2,d
 real(kind=dp)             :: asym_chi

 real(kind=dp)             :: z1,z2
 
 

     if ( dabs(p1)>0.0_dp .and. dabs(p2)>0.0_dp ) then
           z1 = (d/p1)/p1; z2 = (d/p2)/p2
           asym_chi = dlog( dabs(p2/p1)*( (1.0_dp-0.25_dp*z1*(1.0_dp-0.5_dp*z1))/(1.0_dp-0.25_dp*z2*(1.0_dp-0.5_dp*z2)) ) )
     else
           asym_chi = 0.0_dp
     endif

!----------------------------------------------------------------------------------------------------------------------------------!
END FUNCTION asym_chi
!----------------------------------------------------------------------------------------------------------------------------------!


!----------------------------------------------------------------------------------------------------------------------------------!
SUBROUTINE ortho_comp_basis(t1,t2,e1,e2,e3)
!----------------------------------------------------------------------------------------------------------------------------------!

!===purpose===! 

!**********************************************************************************************************************************!
!************************************  Given two linearly independent vectors t1 and t2 in 3D  ************************************!
!** Compute the orthonormal triad {e1,e2,e3} such that e1=t1/norm(t1), norm(e2) = 1 and e2 is orthogonal to e1, and e3 = e1 x e2 **!
!**********************************************************************************************************************************!

!*** Developer: Sylvain Nintcheu Fata, Oak Ridge National Laboratory
!*** Version:   V.1.2
!*** Released:  Apr 14, 2011

 
USE KINDS


IMPLICIT NONE

 real   (kind=dp), intent(in ) :: t1(3),t2(3)
 real   (kind=dp), intent(out) :: e1(3),e2(3),e3(3)

 integer(kind=k9)              :: i
 real   (kind=dp)              :: al,snrm_te,nrm_te,nrm_tx



!*********  Orthonormal local basis: e1(3),e2(3),e3(3)  ***************************************************************************!
    snrm_te = t1(1)*t1(1) + t1(2)*t1(2) + t1(3)*t1(3)
    nrm_te = dsqrt(snrm_te)

    al = (t1(1)*t2(1) + t1(2)*t2(2) + t1(3)*t2(3))/snrm_te
    
    loop1: do i=1,3 ! Loop over all components
       e2(i) = t2(i) - al*t1(i)
    enddo loop1

    nrm_tx = dsqrt(e2(1)*e2(1) + e2(2)*e2(2) + e2(3)*e2(3))

    loop2: do i=1,3 ! Loop over all components
       e1(i) = t1(i)/nrm_te; e2(i) = e2(i)/nrm_tx
    enddo loop2

!*** Unit Normal vector to the plane spanned by {e1,e2}: e3 = e1 x e2
    e3(1) = e1(2)*e2(3) - e2(2)*e1(3)
    e3(2) = e1(3)*e2(1) - e2(3)*e1(1)
    e3(3) = e1(1)*e2(2) - e2(1)*e1(2)

    return

!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE ortho_comp_basis
!----------------------------------------------------------------------------------------------------------------------------------!
